@@ -1,4 +1,10 @@
 # Copyright (C) 2020 FireEye, Inc. All Rights Reserved.
+# Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at: [package root]/LICENSE.txt
+# Unless required by applicable law or agreed to in writing, software distributed under the License
+#  is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and limitations under the License.
 
 import sys
 import builtins
@@ -61,3 +67,18 @@ def generate_api_features(apiname, va):
 
 def all_zeros(bytez):
     return all(b == 0 for b in builtins.bytes(bytez))
+
+
+def twos_complement(val, bits):
+    """
+    compute the 2's complement of int value val
+
+    from: https://stackoverflow.com/a/9147327/87207
+    """
+    # if sign bit is set e.g., 8bit: 128-255
+    if (val & (1 << (bits - 1))) != 0:
+        # compute negative value
+        return val - (1 << bits)
+    else:
+        # return positive value as is
+        return val
